@@ -143,6 +143,25 @@ pango_stretch_get_type (void)
   return etype;
 }
 
+GType
+pango_font_mask_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { PANGO_FONT_MASK_FAMILY, "PANGO_FONT_MASK_FAMILY", "family" },
+      { PANGO_FONT_MASK_STYLE, "PANGO_FONT_MASK_STYLE", "style" },
+      { PANGO_FONT_MASK_VARIANT, "PANGO_FONT_MASK_VARIANT", "variant" },
+      { PANGO_FONT_MASK_WEIGHT, "PANGO_FONT_MASK_WEIGHT", "weight" },
+      { PANGO_FONT_MASK_STRETCH, "PANGO_FONT_MASK_STRETCH", "stretch" },
+      { PANGO_FONT_MASK_SIZE, "PANGO_FONT_MASK_SIZE", "size" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("PangoFontMask", values);
+  }
+  return etype;
+}
+
 
 /* enumerations from "pango-layout.h" */
 GType
