@@ -67,29 +67,6 @@ pango_font_map_load_font  (PangoFontMap               *fontmap,
   return PANGO_FONT_MAP_GET_CLASS (fontmap)->load_font (fontmap, desc);
 }
 
-/**
- * pango_font_map_list_fonts:
- * @fontmap: a #PangoFontMap
- * @family: the family for which to list the fonts, or %NULL
- *          to list fonts in all families.
- * @descs: location to store a pointer to an array of pointers to
- *         #PangoFontDescription. This array should be freed
- *         with pango_font_descriptions_free().
- * @n_descs: location to store the number of elements in @descs
- * 
- * List all fonts in a fontmap, or the fonts in a particular family.
- **/
-void
-pango_font_map_list_fonts (PangoFontMap           *fontmap,
-			   const char             *family,
-			   PangoFontDescription ***descs,
-			   int                    *n_descs)
-{
-  g_return_if_fail (fontmap != NULL);
-
-  PANGO_FONT_MAP_GET_CLASS (fontmap)->list_fonts (fontmap, family, descs, n_descs);
-}
-
 /**
  * pango_font_map_list_families:
  * @fontmap: a #PangoFontMap
@@ -100,9 +77,9 @@ pango_font_map_list_fonts (PangoFontMap           *fontmap,
  * List all families for a fontmap. 
  **/
 void
-pango_font_map_list_families (PangoFontMap   *fontmap,
-			      gchar        ***families,
-			      int            *n_families)
+pango_font_map_list_families (PangoFontMap      *fontmap,
+			      PangoFontFamily ***families,
+			      int               *n_families)
 {
   g_return_if_fail (fontmap != NULL);
 
