@@ -23,20 +23,28 @@
 #define __PANGO_FONTMAP_H__
 
 #include <pango/pango-font.h>
-#include <glib-object.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 #define PANGO_TYPE_FONT_MAP              (pango_font_map_get_type ())
 #define PANGO_FONT_MAP(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FONT_MAP, PangoFontMap))
-#define PANGO_FONT_MAP_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_FONT_MAP, PangoFontMapClass))
 #define PANGO_IS_FONT_MAP(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_TYPE_FONT_MAP))
+
+typedef struct _PangoFontMap PangoFontMap;
+
+GType      pango_font_map_get_type       (void) G_GNUC_CONST;
+PangoFont *pango_font_map_load_font     (PangoFontMap                 *fontmap,
+					 const PangoFontDescription   *desc);
+void       pango_font_map_list_families (PangoFontMap                 *fontmap,
+					 PangoFontFamily            ***families,
+					 int                          *n_families);
+
+#ifdef PANGO_ENABLE_BACKEND
+
+#define PANGO_FONT_MAP_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_FONT_MAP, PangoFontMapClass))
 #define PANGO_IS_FONT_MAP_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), PANGO_TYPE_FONT_MAP))
 #define PANGO_FONT_MAP_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), PANGO_TYPE_FONT_MAP, PangoFontMapClass))
 
-typedef struct _PangoFontMap PangoFontMap;
 typedef struct _PangoFontMapClass PangoFontMapClass;
 
 struct _PangoFontMap
@@ -50,30 +58,13 @@ struct _PangoFontMapClass
   
   PangoFont *(*load_font)     (PangoFontMap               *fontmap,
 			       const PangoFontDescription *desc);
-  void       (*list_fonts)    (PangoFontMap               *fontmap,
-			       const gchar                *family,
-			       PangoFontDescription     ***descs,
-			       int                        *n_descs);
   void       (*list_families) (PangoFontMap               *fontmap,
-			       gchar                    ***families,
+			       PangoFontFamily          ***families,
 			       int                        *n_families);
 };
 
-GType      pango_font_map_get_type       (void) G_GNUC_CONST;
-PangoFont *pango_font_map_load_font     (PangoFontMap                 *fontmap,
-					 const PangoFontDescription   *desc);
-void       pango_font_map_list_fonts    (PangoFontMap                 *fontmap,
-					 const gchar                  *family,
-					 PangoFontDescription       ***descs,
-					 int                          *n_descs);
-void       pango_font_map_list_families (PangoFontMap                 *fontmap,
-					 gchar                      ***families,
-					 int                          *n_families);
-void       pango_font_map_free_families (gchar                       **families,
-					 int                           n_families);
+#endif /* PANGO_ENABLE_BACKEND */
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __PANGO_FONTMAP_H__ */
