@@ -48,8 +48,8 @@ typedef struct _PangoFT2ContextInfo PangoFT2ContextInfo;
 
 struct _PangoFT2MetricsInfo
 {
-  const char *lang;
-  PangoFontMetrics metrics;
+  const char *sample_str;
+  PangoFontMetrics *metrics;
 };
 
 struct _PangoFT2FontClass
@@ -61,16 +61,16 @@ static PangoFontClass *parent_class;	/* Parent class structure for PangoFT2Font
 
 static void pango_ft2_font_class_init (PangoFT2FontClass *class);
 static void pango_ft2_font_init       (PangoFT2Font      *xfont);
-static void pango_ft2_font_shutdown   (GObject         *object);
+static void pango_ft2_font_dispose    (GObject         *object);
 static void pango_ft2_font_finalize   (GObject         *object);
 
 static PangoFontDescription *pango_ft2_font_describe          (PangoFont            *font);
 
 static PangoCoverage *       pango_ft2_font_get_coverage      (PangoFont            *font,
-							       const char           *lang);
+							       PangoLanguage        *language);
 
 static PangoEngineShape *    pango_ft2_font_find_shaper       (PangoFont            *font,
-							       const char           *lang,
+							       PangoLanguage        *language,
 							       guint32               ch);
 
 static void                  pango_ft2_font_get_glyph_extents (PangoFont            *font,
@@ -78,9 +78,8 @@ static void                  pango_ft2_font_get_glyph_extents (PangoFont
 							       PangoRectangle       *ink_rect,
 							       PangoRectangle       *logical_rect);
 
-static void                  pango_ft2_font_get_metrics       (PangoFont            *font,
-							       const gchar          *lang,
-							       PangoFontMetrics     *metrics);
+static PangoFontMetrics *    pango_ft2_font_get_metrics       (PangoFont            *font,
+							       PangoLanguage        *language);
   
 static void                  pango_ft2_get_item_properties    (PangoItem      	    *item,
 							       PangoUnderline 	    *uline,
@@ -224,7 +223,7 @@ pango_ft2_font_class_init (PangoFT2FontClass *class)
   parent_class = g_type_class_peek_parent (class);
   
   object_class->finalize = pango_ft2_font_finalize;
-  object_class->shutdown = pango_ft2_font_shutdown;
+  object_class->dispose = pango_ft2_font_dispose;
   
   font_class->describe = pango_ft2_font_describe;
   font_class->get_coverage = pango_ft2_font_get_coverage;
@@ -351,14 +350,25 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 		  {
 		    p = bitmap->buffer +
 		      (iyoff - face->glyph->bitmap_top + iy) * bitmap->pitch +
-		      ixoff +
-		      face->glyph->bitmap_left + x_start;
+		      ixoff + face->glyph->bitmap_left + 
+                      x_start;
 		    
-		    q = face->glyph->bitmap.buffer + iy*face->glyph->bitmap.pitch;
+		    q = face->glyph->bitmap.buffer + 
+                      iy * face->glyph->bitmap.pitch;
+
 		    for (ix = x_start; ix < x_limit; ix++)
 		      {
-			*p = *q;
-			q++;
+                        switch (*q)
+                          {
+                          case 0:
+                            break;
+                          case 0xff:
+                            *p = 0xff;
+                          default:
+                            *p = MIN ((gushort) *p + (gushort) *q, 0xff);
+                            break;
+                          }
+                        q++;
 			p++;
 		      }
 		  }
@@ -367,17 +377,17 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 		  {
 		    p = bitmap->buffer +
 		      (iyoff - face->glyph->bitmap_top + iy) * bitmap->pitch +
-		      ixoff +
-		      face->glyph->bitmap_left + x_start;
-		    
-		    q = face->glyph->bitmap.buffer + iy*face->glyph->bitmap.pitch;
+		      ixoff + face->glyph->bitmap_left + 
+                      x_start;
+                    
+                    q = face->glyph->bitmap.buffer + 
+                      iy*face->glyph->bitmap.pitch;
+
 		    for (ix = x_start; ix < x_limit; ix++)
 		      {
-			if ((*q) & (1 << (7 - (ix%8))))
-			  *p = 0;
-			else
-			  *p = MIN (*p, 0xFF);
-			if ((ix%8) == 7)
+			if ((*q) & (1 << (7 - (ix % 8))))
+			  *p = 0xff;
+			if ((ix % 8) == 7)
 			  q++;
 			p++;
 		      }
@@ -553,7 +563,7 @@ get_font_metrics_from_subfonts (PangoFont        *font,
  */
 static void
 get_font_metrics_from_string (PangoFont        *font,
-			      const char       *lang,
+			      PangoLanguage    *language,
 			      const char       *str,
 			      PangoFontMetrics *metrics)
 {
@@ -562,11 +572,11 @@ get_font_metrics_from_string (PangoFont        *font,
   PangoEngineShape *shaper, *last_shaper;
   int last_level;
   gunichar *text_ucs4;
-  int n_chars, i;
+  long n_chars, i;
   guint8 *embedding_levels;
   PangoDirection base_dir = PANGO_DIRECTION_LTR;
   GSList *subfonts = NULL;
-  
+
   text_ucs4 = g_utf8_to_ucs4_fast (str, -1, &n_chars);
   if (!text_ucs4)
     return;
@@ -586,7 +596,7 @@ get_font_metrics_from_string (PangoFont        *font,
       gunichar wc = g_utf8_get_char (p);
       p = g_utf8_next_char (p);
 	  
-      shaper = pango_font_find_shaper (font, lang, wc);
+      shaper = pango_font_find_shaper (font, language, wc);
       if (p > start &&
 	  (shaper != last_shaper || last_level != embedding_levels[i]))
 	{
@@ -620,98 +630,26 @@ get_font_metrics_from_string (PangoFont        *font,
   
   pango_glyph_string_free (glyph_str);
   g_free (embedding_levels);
+
+  return;
 }
 
-typedef struct {
-  const char *lang;
-  const char *str;
-} LangInfo;
-
-int
-lang_info_compare (const void *key,
-		   const void *val)
-{
-  const LangInfo *lang_info = val;
-
-  return strncmp (key, lang_info->lang, 2);
-}
-
-/* The following array is supposed to contain enough text to tickle all necessary fonts for each
- * of the languages in the following. Yes, it's pretty lame. Not all of the languages
- * in the following have sufficient text to excercise all the accents for the language, and
- * there are obviously many more languages to include as well.
- */
-LangInfo lang_texts[] = {
-  { "ar", "Arabic  ا,DXXم عY(JY,CY(B" },
-  { "cs", "Czech (česky)  Dobrý den" },
-  { "da", "Danish (Dansk)  Hej, Goddag" },
-  { "el", "Greek ($(GN;ληνικά(B) $(CN5ιά ,CNς(B" },
-  { "en", "English Hello" },
-  { "eo", "Esperanto Saluton" },
-  { "es", "Spanish (Español) ¡Hola!" },
-  { "et", "Estonian  Tere, Tervist" },
-  { "fi", "Finnish (Suomi)  Hei, Hyvää päivää" },
-  { "fr", "French (Français)" },
-  { "de", "German Grüß Gott" },
-  { "iw", "Hebrew   שלום" },
-  { "il", "Italiano  Ciao, Buon giorno" },
-  { "ja", "Japanese (日語) こん, ｺﾝ,Foﾊ(B" },
-  { "ko", "Korean (한글)   ,Hk하,D8요(B, ,Hk하십j(B" },
-  { "mt", "Maltese   Ċaw, Saħħa" },
-  { "nl", "Nederlands, Vlaams Hallo, Dag" },
-  { "no", "Norwegian (Norsk) Hei, God dag" },
-  { "pl", "Polish   Dzień dobry, Hej" },
-  { "ru", "Russian (Р,CQкий(B)" },
-  { "sk", "Slovak   Dobrý deň" },
-  { "sv", "Swedish (Svenska) Hej på dej, Goddag" },
-  { "tr", "Turkish (Türkçe) Merhaba" },
-  { "zh", "Chinese (中文,$(1.i话(B,(Ih(B)" }
-};
-
-static void
+static PangoFontMetrics *
 pango_ft2_font_get_metrics (PangoFont        *font,
-			    const gchar      *lang,
-			    PangoFontMetrics *metrics)
+			    PangoLanguage    *language)
 {
-  PangoFT2MetricsInfo *info;
+  PangoFT2MetricsInfo *info = NULL; /* Quiet GCC */
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
   GSList *tmp_list;
       
-  const char *lookup_lang;
-  const char *str;
-
-  if (lang)
-    {
-      LangInfo *lang_info = bsearch (lang, lang_texts,
-				     G_N_ELEMENTS (lang_texts), sizeof (LangInfo),
-				     lang_info_compare);
-
-      if (lang_info)
-	{
-	  lookup_lang = lang_info->lang;
-	  str = lang_info->str;
-	}
-      else
-	{
-	  lookup_lang = "UNKNOWN";
-	  str = "French (Français)";     /* Assume iso-8859-1 */
-	}
-    }
-  else
-    {
-      lookup_lang = "NONE";
-
-      /* Complete junk
-       */
-      str = "ا,DXXم عY(JY,CY č(Besky $(GN;ληνικά (BFrançais 日語 한글 Р,CQкий 中文(B,$(1.i话(B,(Ih (BTürkçe";
-    }
+  const char *sample_str = pango_language_get_sample_string (language);
   
   tmp_list = ft2font->metrics_by_lang;
   while (tmp_list)
     {
       info = tmp_list->data;
       
-      if (info->lang == lookup_lang)        /* We _don't_ need strcmp */
+      if (info->sample_str == sample_str)        /* We _don't_ need strcmp */
 	break;
 
       tmp_list = tmp_list->next;
@@ -720,14 +658,14 @@ pango_ft2_font_get_metrics (PangoFont        *font,
   if (!tmp_list)
     {
       info = g_new (PangoFT2MetricsInfo, 1);
-      info->lang = lookup_lang;
+      info->sample_str = sample_str;
+      info->metrics = pango_font_metrics_new ();
+      get_font_metrics_from_string (font, language, sample_str, info->metrics);
 
       ft2font->metrics_by_lang = g_slist_prepend (ft2font->metrics_by_lang, info);
-      
-      get_font_metrics_from_string (font, lang, str, &info->metrics);
     }
       
-  *metrics = info->metrics;
+  return pango_font_metrics_ref (info->metrics);
 }
 
 /**
@@ -749,8 +687,8 @@ pango_ft2_n_subfonts (PangoFont *font)
 }
 
 PangoCoverage *
-pango_ft2_get_coverage (PangoFont  *font,
-			const char *lang)
+pango_ft2_get_coverage (PangoFont     *font,
+			PangoLanguage *language)
 {
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
   PangoCoverage *result = pango_coverage_new ();
@@ -809,7 +747,7 @@ pango_ft2_font_subfont_open_args (PangoFont        *font,
 }
 
 static void
-pango_ft2_font_shutdown (GObject *object)
+pango_ft2_font_dispose (GObject *object)
 {
   PangoFT2Font *ft2font = PANGO_FT2_FONT (object);
 
@@ -820,7 +758,7 @@ pango_ft2_font_shutdown (GObject *object)
   if (!ft2font->in_cache && ft2font->fontmap)
     pango_ft2_fontmap_cache_add (ft2font->fontmap, ft2font);
 
-  G_OBJECT_CLASS (parent_class)->shutdown (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
@@ -831,6 +769,13 @@ pango_ft2_free_glyph_info_callback (gpointer key, gpointer value, gpointer data)
   return TRUE;
 }
 
+static void
+free_metrics_info (PangoFT2MetricsInfo *info)
+{
+  pango_font_metrics_unref (info->metrics);
+  g_free (info);
+}
+
 static void
 pango_ft2_font_finalize (GObject *object)
 {
@@ -849,11 +794,11 @@ pango_ft2_font_finalize (GObject *object)
   g_free (ft2font->oa);
   g_free (ft2font->faces);
 
-  g_slist_foreach (ft2font->metrics_by_lang, (GFunc)g_free, NULL);
+  g_slist_foreach (ft2font->metrics_by_lang, (GFunc)free_metrics_info, NULL);
   g_slist_free (ft2font->metrics_by_lang);
   
   if (ft2font->entry)
-    pango_ft2_font_entry_remove (ft2font->entry, (PangoFont *)ft2font);
+    pango_ft2_face_remove (ft2font->entry, (PangoFont *)ft2font);
 
   g_object_unref (G_OBJECT (ft2font->fontmap));
 
@@ -871,14 +816,14 @@ pango_ft2_font_describe (PangoFont *font)
 
   ft2font = PANGO_FT2_FONT (font);
 
-  desc = pango_font_description_copy (&ft2font->entry->description);
-  desc->size = ft2font->size;
+  desc = pango_font_description_copy (ft2font->entry->description);
+  pango_font_description_set_size (desc, ft2font->size);
 
   return desc;
 }
 
 PangoMap *
-pango_ft2_get_shaper_map (const char *lang)
+pango_ft2_get_shaper_map (PangoLanguage *language)
 {
   static guint engine_type_id = 0;
   static guint render_type_id = 0;
@@ -889,26 +834,26 @@ pango_ft2_get_shaper_map (const char *lang)
       render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_FT2);
     }
   
-  return pango_find_map (lang, engine_type_id, render_type_id);
+  return pango_find_map (language, engine_type_id, render_type_id);
 }
 
 static PangoCoverage *
-pango_ft2_font_get_coverage (PangoFont  *font,
-			     const char *lang)
+pango_ft2_font_get_coverage (PangoFont     *font,
+			     PangoLanguage *language)
 {
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
 
-  return pango_ft2_font_entry_get_coverage (ft2font->entry, font, lang);
+  return pango_ft2_face_get_coverage (ft2font->entry, font, language);
 }
 
 static PangoEngineShape *
-pango_ft2_font_find_shaper (PangoFont   *font,
-			    const gchar *lang,
-			    guint32      ch)
+pango_ft2_font_find_shaper (PangoFont     *font,
+			    PangoLanguage *language,
+			    guint32        ch)
 {
   PangoMap *shape_map = NULL;
 
-  shape_map = pango_ft2_get_shaper_map (lang);
+  shape_map = pango_ft2_get_shaper_map (language);
   return (PangoEngineShape *)pango_map_get_engine (shape_map, ch);
 }
 
@@ -1107,7 +1052,7 @@ pango_ft2_get_item_properties (PangoItem      *item,
 			       PangoAttrColor *bg_color,
 			       gboolean       *bg_set)
 {
-  GSList *tmp_list = item->extra_attrs;
+  GSList *tmp_list = item->analysis.extra_attrs;
 
   if (fg_set)
     *fg_set = FALSE;
