@@ -25,9 +25,7 @@
 
 #include <pango/pango-layout.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 #include <freetype/freetype.h>
 
@@ -65,15 +63,15 @@ typedef guint16 PangoFT2Subfont;
 #define PANGO_FT2_GLYPH_SUBFONT(glyph) ((glyph)>>16)
 #define PANGO_FT2_GLYPH_INDEX(glyph) ((glyph) & 0xFFFF)
 
-int            pango_ft2_n_subfonts        (PangoFont  *font);
-PangoGlyph     pango_ft2_get_unknown_glyph (PangoFont  *font);
-int            pango_ft2_font_get_kerning  (PangoFont  *font,
-					    PangoGlyph  left,
-					    PangoGlyph  right);
-PangoCoverage *pango_ft2_get_coverage      (PangoFont  *font,
-					    const char *lang);
-FT_Face        pango_ft2_get_face          (PangoFont      *font,
-					    PangoFT2Subfont subfont_index);
+int            pango_ft2_n_subfonts        (PangoFont       *font);
+PangoGlyph     pango_ft2_get_unknown_glyph (PangoFont       *font);
+int            pango_ft2_font_get_kerning  (PangoFont       *font,
+					    PangoGlyph       left,
+					    PangoGlyph       right);
+PangoCoverage *pango_ft2_get_coverage      (PangoFont       *font,
+					    PangoLanguage   *language);
+FT_Face        pango_ft2_get_face          (PangoFont       *font,
+					    PangoFT2Subfont  subfont_index);
 
 /* API for libraries that want to use PangoFT2 mixed with classic
  * FT2 fonts.
@@ -101,9 +99,6 @@ void               pango_ft2_font_subfont_open_args  (PangoFont         *font,
 void     pango_ft2_fontmap_dump (int           indent,
 				 PangoFontMap *fontmap);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __PANGOFT2_H__ */
